/*
 * @copyright       © Copyright 2006-2010, epages GmbH, All Rights Reserved.
 *
 * @module          de_epages/product/inc/energylabelSettings
 *
 * @revision        $Revision: 1.2.32.1 $
 */
/*global define*/
/*jslint browser: true, regexp:true, unparam: true*/

define('de_epages/product/inc/energylabelsettings', [
    'jquery',
    "$tmpl!./energylabelsettings-scale",

    'jquery/ui/slider',
    'ep/ui/slider',
    '$ready!'
], function ($, tmplScale) {
    'use strict';

    return {
        init: function (opts) {
            var self = this,
                o;

            // extend default values
            self.opts = $.extend({
                labelsIsDefined: false,
                sourceIsDefined: false,
                numberOfLabels: 0,
                numberOfSystemLabels: 0,
                fromTo: '',
                labelMap: {},
                wrapper: $('body'),
                sourceInput: undefined
            }, opts);

            o = self.opts;
            o.fromTo = o.fromTo ? o.fromTo.split('-').map(function (str) {
                return str.trim();
            }) : [];

            self.$wrapper = o.wrapper;
            self.singleSlider = self.$wrapper.find('#EngergyLabelSingleSlider');
            self.rangeSlider = self.$wrapper.find('#EngergyLabelRangeSlider');

            self.form = self.$wrapper.closest('form');
            // Observe the file input field
            o.sourceInput.on('change', function(evt) {
                self._change(evt.target.files);
            });

            o.sourceInput.on('delete', function (evt) {
                // Update values
                o.sourceIsDefined = false;
                o.labelsIsDefined = false;
                o.numberOfLabels = 0;


                // Energylabel file was deleted, so we delete also the label class
                self.$wrapper.find('#EnergyLabelStart').val('');
                self.$wrapper.find('#EnergyLabelEnd').val('');

                o.sourceInput.trigger('change');
            });

            // Observe the radio buttons
            self.$wrapper.find('input[name=EnergyLabelSingleOrRange]').on('change', function (evt) {
                if (this.checked) {
                    if (Number(this.value) === 0) {
                        // Show single slider
                        self._toggleSlider(self.singleSlider, self.rangeSlider);
                    } else {
                        // Show range slider
                        self._toggleSlider(self.rangeSlider, self.singleSlider);
                    }
                }
            });

            // Initialize sliders
            self._initSlider(self.singleSlider, true);
            self._initSlider(self.rangeSlider, false);

            // wait for dimensions of slider
            window.setTimeout(function () {
                self._createScale(self.singleSlider);
            }, 150);

            // Setup
            self._change([]);

        },

        _change: function (files) {
            var self = this,
                o = self.opts,
                fileSelected = o.sourceIsDefined || !!files.length,
                isRange = ((o.labelsIsDefined && o.numberOfLabels > 1) || self.$wrapper.find('#RangeLabel')[0].checked) ? true : false;

            // Enable/Disable energylabel functionality
            self.$wrapper.find('input').prop('disabled', !fileSelected);
            self.singleSlider.slider('option', 'disabled', !fileSelected);
            self.rangeSlider.slider('option', 'disabled', !fileSelected);
            self.$wrapper.toggle(!!fileSelected);

            if (fileSelected) {
                if (isRange) {
                    self._toggleSlider(self.rangeSlider, self.singleSlider);
                } else {
                    self._toggleSlider(self.singleSlider, self.rangeSlider);
                }
            }
        },

        _createScale: function (singleSlider) {
            var self = this,
                o = self.opts,
                sliderWidth = singleSlider.width(),
                part = sliderWidth / (o.labelMap.labels.length - 1),
                tmpl = {
                    labels: o.labelMap.labels,
                    width: part,
                    half: part / 2,
                    length: o.labelMap.labels.length
                };

            // render template for scale
            tmplScale(tmpl).appendTo(o.wrapper.find('#EnergyLabelRangeWrapper'));
        },
        _initSlider: function (slider, single) {
            var self = this,
                o = self.opts,
                inputStart = o.wrapper.find('#EnergyLabelStart'),
                inputEnd = o.wrapper.find('#EnergyLabelEnd'),
                labelMap = o.labelMap,
                labelsIsDefined = o.labelsIsDefined,
                numberOfLabels = o.numberOfLabels,
                fromTo = o.fromTo;

            slider.slider({
                disabled: true,
                min: 0,
                max: o.numberOfSystemLabels - 1,
                step: 1,
                range: single ? false : true,
                value: (single && labelsIsDefined && numberOfLabels === 1) ? labelMap[fromTo[0]] : 0,
                values: !single ? ((labelsIsDefined && numberOfLabels > 1) ? [labelMap[fromTo[0]], labelMap[fromTo[1]]] : [0, o.numberOfSystemLabels - 1]) : null,
                start: function (event, ui) {
                    // trigger change to set the save button active if a form around the energy label wrapper exists
                    if (self.form.length) {
                        self.form.trigger('change');
                    }
                },
                slide: single
                    ? function (evt, ui) {
                        self._updateArrow(labelMap[ui.value].class, labelMap[ui.value].text);
                    }
                    : function (evt, ui) {
                        var label;

                        if (ui.values[1] - ui.values[0] === 0) {
                            return false;
                        }

                        label = self._getMedian(ui.values[0], ui.values[1]);
                        self._updateArrow(labelMap[label].class, labelMap[ui.values[0]].text + ' - ' + labelMap[ui.values[1]].text);
                    },
                stop: single
                    ? function (evt, ui) {
                        // Disable second input
                        inputEnd.prop('disabled', true);

                        inputStart.val(labelMap[ui.value].text);
                        inputEnd.val('');

                    }
                    : function (evt, ui) {
                        // Enable second input
                        inputEnd.prop('disabled', false);

                        inputStart.val(labelMap[ui.values[0]].text);
                        inputEnd.val(labelMap[ui.values[1]].text);

                    }
            });
        },
        _updateArrow: function (labelClass, text) {
            var arrow = this.opts.wrapper.find('.energylabel-arrow'),
                old = arrow.attr('class').replace(/\sclass-./g, '');

            arrow.attr('class', old);
            arrow.addClass(labelClass);
            arrow.html(text);
        },

        // Attention: Returns position of the median inside the labelmap
        // This is not the energy class!
        // `start` and `end` are the positions of the slider handles which start with 0
        _getMedian: function (start, end) {
            var median = Math.ceil((end + start) / 2);
            return median > 7 ? 7 : median;
        },
        _toggleSlider: function (enableSlider, disableSlider) {
            var self = this,
                o = self.opts,
                values = enableSlider.slider('values'),
                value = enableSlider.slider('value'),
                isRange = values.length,
                position = isRange ? self._getMedian(values[0], values[1]) : value,
                text = isRange ? o.labelMap[values[0]].text + ' - ' + o.labelMap[values[1]].text : o.labelMap[position].text;

            self._updateArrow(o.labelMap[position].class, text);
            enableSlider
                .show()
                .slider('enable');
            disableSlider
                .hide()
                .slider('disable');

            // Set the hidden inputs correctly
            enableSlider.slider('option', 'stop').call(enableSlider, '', isRange ? {values: values} : {value: value});
        }
    };
});