/*jslint nomen:true*/
/*global define, window*/

/*
 * @copyright       © Copyright 2006-2014, epages GmbH, All Rights Reserved.
 *
 * @module          de_epages/product/inc/energylabel
 *
 * @revision        $Revision: 1.2.26.1 $
 *
 * The energylabel function creates a lightbox with all found energy labels.
 *
 * ### Example
 * HTML:
 *
 *     <div class="energylabel-arrow left" data-energylabel-source="#EnergyLabelURL">A++</div>
 *
 *
 * @chainable
 * @return {Object} jQuery
 */
define('de_epages/product/inc/energylabel', [
    'jquery',

    'ep/ui/lightbox'
], function ($) {

    'use strict';

    var list;

    return {
        init: function (opts) {
            var options = $.extend({
                    selector: 'energylabel-imagelist',
                    items: [],
                    body: $('body')
                }, opts),
                self = this,
                /*
                 * Check if the media source is a pdf or an image. Create the necessary element and register events to controll the widget.
                 *
                 * @since 6.17.12
                 * @private
                 */
                 _create = function () {
                    var hasImages = false;

                    options.items.each(function (index, elem) {
                        var $elem = $(elem),
                            src = $elem.data('energylabel-source'),
                            image = $('<img alt src="' + src + '"/>'),
                            // Get the file extension
                            isPdf = src.substring(src.lastIndexOf('.') + 1) === 'pdf';

                        if (isPdf) {
                            $elem.on('click', function () {
                                // Open pdf in new window
                                window.open(src, '_blank');
                            });
                        } else {
                            hasImages = true;
                            list.append($('<li>').append(image));
                            $elem.on('click', function () {
                                image.trigger('click');
                            });
                        }
                    });

                    if (hasImages) {
                        self._buildLightbox({
                            images: '#' + options.selector + ' img',
                            controls: 'hidden'
                        });
                    }
                };

            list = $('<ul style="display:none;" id="' + options.selector + '" />').appendTo(options.body),
            _create();

            return $;
        },
        _buildLightbox: function (opts) {
            list.uiLightbox(opts);
        }
    }
});
