/*globals define*/
define([
    'jquery',
    'testsuite/qunit',
    'testsuite/qunit-fixture!de_epages/product/inc/energylabel-test.html',
    'testsuite/sinon',
    'de_epages/product/inc/energylabel',

    'ep/ui/lightbox',
    '$ready!'
], function ($, Q, fixture, sinon, energylabel) {
    'use strict';

    var lightboxStub,
        windowStub;

    QUnit.module("ep/ui/lightbox", {
        setup: function () {
            lightboxStub = sinon.stub(energylabel, '_buildLightbox');
            windowStub = sinon.stub(window, 'open');
            fixture.init();
            energylabel.init({
                items: $('.energylabel-arrow'),
                body: $('#qunit-fixture')}
            );
        },
        teardown: function () {
            lightboxStub.restore();
            windowStub.restore();
            fixture.destroy();
        }
    });


    Q.test('Check HTML structure and visible options', function () {
        Q.ok(
            $('#energylabel-imagelist').length,
            'List was created'
        );

        Q.strictEqual(
            $('#energylabel-imagelist li').length,
            2,
            'Two images were inserted'
        );

        Q.equal(
            lightboxStub.calledWith({images: "#energylabel-imagelist img", controls: "hidden"}),
            true,
            'Corrct parameters are passed to the lightbox'
        );
    });


    Q.asyncTest('Open image in lightbox', function () {
        $('.foo').trigger('click');

        setTimeout(function () {
            Q.equal(
               windowStub.called,
                false,
                'Image is not opened in new window'
            );

            Q.start();
        }, 300);

    });

    Q.asyncTest('Open pdf in new window', function () {
        $('.pdf').trigger('click');

        setTimeout(function () {
            Q.equal(
                windowStub.called,
                true,
                'Corrct parameters are passed to window.open'
            );

            Q.start();
        }, 300);
    });

});